# -*- coding: utf-8 -*-
#
# Copyright 2017 Spotify AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from ulogger.ulogger import setup_logging

__author__ = 'Lynn Root'
__version__ = '1.0.0'
__license__ = 'Apache 2.0'
__email__ = 'lynn@spotify.com'
__description__ = 'Micro logging library'
__uri__ = 'https://github.com/spotify/ulogger'

__all__ = ['setup_logging']
